/*
Jed Margolin 7/4/2024
pico-rx-test
Receive nrf905 data packet and display the raw data on the LCD and use uart
LCD display first 10 bytes of the packet, uart sends all 32 bytes
Assumes at least 1 second between packets
Uses rpi-pico-v6 board  
   DS3231 RTC
   nrf905 RX
   LCD
   ADC not used
*/

#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "main.h"
#include "hardware/uart.h"
#include <string.h>

#define ADC 0      // 1 = ADC Preamp is loaded (but not used here), 0 = ADC Preamp is not loaded
//================================================================
bool repeating_timer_callback(struct repeating_timer *t) 
{
   if(mode)     //  For RX   1 ms
   {
      if(gpio_get(DR) && gpio_get(AM) )    // Data received by nrf905
      { 
         gpio_put(LED,1);   // data received, turn off later

      // Get the data from nrf905
      // Initialize output data buffer; first byte is Read RX Payload command
         spi_out_buffer[0] = 0x24;

      // Get the RX data
         gpio_put(CSN905,0);   
      // Write the output buffer to MOSI, and at the same time read from MISO.
         spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, BUF_LEN);
         gpio_put(CSN905,1); 

         data_received = 1;  
      }
      
      // Not used by this program
      irq_count--;
      if(irq_count == 0)
      {
         irq_count = 1000;   // 1000 = 1 second
         secs_flag = 1;
      }

   }
   else   // for ds3231 clock   4ms
   {
      irq_flag = 1;
      // read switches
      do_switch(PB1, &switch1);   // PB1, switch structure  INC
      do_switch(PB2, &switch2);   // PB2, switch structure  DEC
      do_switch(PB3, &switch3);   // PB3, switch structure  MODE
   
      irq_count--;
      if(irq_count == 0)
      {
         irq_count = 250;   // 250 = 1 second
      }
   }

   return true;
}
//===============================================================
int main () 
{
   gpio_init(LED);   // LED
   gpio_set_dir(LED, GPIO_OUT);
   gpio_put(LED,0);

   gpio_init(PB1);     // Pushbutton 1
   gpio_set_dir(PB1, GPIO_IN);
   gpio_pull_up(PB1);

   gpio_init(PB2);     // Pushbutton 2
   gpio_set_dir(PB2, GPIO_IN);
   gpio_pull_up(PB2);

   gpio_init(PB3);     // Pushbutton 3
   gpio_set_dir(PB3, GPIO_IN);
   gpio_pull_up(PB3);

   gpio_init(SW);     // Slide Switch for Mode (RX or Clock)
   gpio_set_dir(SW, GPIO_IN);
   gpio_pull_up(SW);

   gpio_init(CHANNEL);     // nrf905 Channel
   gpio_set_dir(CHANNEL, GPIO_IN);
   gpio_pull_up(CHANNEL);
//-------------------
// ds3231 uses I2C0 on the default SDA and SCL pins (4, 5)
   i2c_init(i2c_default, 300 * 1000);   // 300 KHz (400 KHz nax)
   gpio_set_function(PICO_DEFAULT_I2C_SDA_PIN, GPIO_FUNC_I2C);
   gpio_set_function(PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C);
   gpio_pull_up(PICO_DEFAULT_I2C_SDA_PIN);
   gpio_pull_up(PICO_DEFAULT_I2C_SCL_PIN);

// Make the I2C pins available to picotool
//    bi_decl(bi_2pins_with_func(PICO_DEFAULT_I2C_SDA_PIN, PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C));
//-------------------
// nrf905 uses SPI-0
// Enable SPI 0 at 5 MHz and connect to GPIOs
   spi_init(spi_default, 5000 * 1000);
   gpio_set_function(PICO_DEFAULT_SPI_RX_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_SCK_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_TX_PIN, GPIO_FUNC_SPI);

   gpio_init(TXEN);    
   gpio_set_dir(TXEN, GPIO_OUT);
   gpio_put(TXEN,0);   // nrf905 Transmit Enable = High

   gpio_init(TRX_CE);    
   gpio_set_dir(TRX_CE, GPIO_OUT);
   gpio_put(TRX_CE,1);   // nrf905 RX Mode
   sleep_us(1000);

   gpio_init(DR);    
   gpio_set_dir(DR, GPIO_IN);
//   gpio_pull_up(DR);   // default is pull down

   gpio_init(AM);   // address match    
   gpio_set_dir(AM, GPIO_IN);
//   gpio_pull_up(AM);  // default is pull down

   gpio_init(CSN905);   // nrf905 CSN
   gpio_set_dir(CSN905, GPIO_OUT);
   gpio_put(CSN905,1);   

// RX mode
   gpio_put(TXEN,0);
   sleep_us(20);
   gpio_put(TRX_CE,1);

// Make the SPI pins available to picotool
//   bi_decl(bi_4pins_with_func(PICO_DEFAULT_SPI_RX_PIN, PICO_DEFAULT_SPI_TX_PIN, PICO_DEFAULT_SPI_SCK_PIN, PICO_DEFAULT_SPI_CSN_PIN, GPIO_FUNC_SPI));

// CLK idle state = low, data sampled on rising edge and shifted on falling edge
// Does not support LSB first; fortunately the nrf905 is MSB first  
   spi_set_format(spi_default, 8, 0, 0, SPI_MSB_FIRST);  
//-------------------
// Set up the UART with the required speed.
   uart_init(UART_ID, BAUD_RATE);

// Set the TX and RX pins by using the function select on the GPIO
   gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
   gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);
//-------------------
#if ADC
// ADC Preamp is loaded but not used)
   adc_init();
   adc_gpio_init(ADC0_PIN);    // Make sure GPIO is high-impedance, no pullups etc
   adc_select_input(0);  // Select ADC input 0 (GPIO-26, pin 31)
#else
// ADC Preamp is not loaded, make it a digital input
   gpio_init(ADC0_PIN);
   gpio_set_dir(ADC0_PIN, GPIO_IN);
   gpio_pull_up(ADC0_PIN);
#endif
//-------------------
   lcd_init();

// Set some things
   mode = gpio_get(SW);
   irq_flag = 0;
   irq_count = 250;
   secs_flag = 0;
   data_received = 0;

 // Greetings
   domsg(&rx_msg1);   // "Jed Margolin's"
   domsg(&rx_msg2);   // "RX Test"

   sleep_ms(2000); 
   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen
   domsg(&rx_msg3);   // "RX [date]"

   sleep_ms(1000); 
   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

   nrf905_init();
//=====================================
// Create a repeating timer that calls repeating_timer_callback
// If the delay is negative the next callback will be starting from the beginning of the callback
// Positive starts from the end of the callback
//   add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  //  4 ms

   if(mode)   
   {
      add_repeating_timer_ms(-1,repeating_timer_callback, NULL, &timer);  // 1 ms for RX
      nrf905_rx();  // Receive and display data; doesn't come back
   }
   else
   {
      add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  // 4 ms for clock
      do_clock();   // Display and set the ds3231; doesn't come back
   }

} // end main
//======================================================================
void nrf905_rx()
{

// Send the title message and version out through uart
   strcpy(line, version);
   strcat(line, "\n");
   uart_puts(UART_ID, line);   // send it out

   SetCmdMode();     // set lcd for commands
   lcd_byte(0x01);   // clear screen
   domsg(&rx_msg4);  // "Waiting For Data"

   while(1)
   {
      if(data_received)    // Data received
      { 
         data_received = 0;
    
         // Display the data
         show_data_lcd();  // send first 10 bytes to LCD
         log_data_ts();    // send data out through uart for logging the data, 32 hex bytes
         gpio_put(LED,0);  // now turn off the LED
      }  
    
      if( !gpio_get(PB1) | !gpio_get(PB2) | !gpio_get(PB3) )   { log_data_ts();}

   }  // end while

}  // end of function
//===================================================
// Solves the power-up problem by doing a dummy read of the RX Buffer
void nrf905_init()
{
   unsigned char n, lcd_pos;

// Writing the Configuration and TXADR address
   if( gpio_get(CHANNEL) )  
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC2, 11);     // write Configuration  912.2 MHz
      gpio_put(CSN905,1);

      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR2, 5);
      gpio_put(CSN905,1);
   }
   else 
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC1, 11);   // write Configuration  433.4 MHz  Test
      gpio_put(CSN905,1);
     
      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR1, 5);
      gpio_put(CSN905,1);
   }

   sleep_ms(100);

// Reading the Configuration
   spi_out_buffer[0] = 0x10;   // command for reading Configuration

   gpio_put(CSN905,0);   
   spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, 11);
   gpio_put(CSN905,1);   

// Display the Configuration data
   lcd_pos = 0x80;
   for(n=1; n<11; n++)
   {
      lcd_wr_hex(lcd_pos,spi_in_buffer[n]);
      lcd_pos = lcd_pos + 3;

      if(n == 5) {lcd_pos = 0xC0;}
   }
   
   sleep_ms(2000);

   gpio_put(TXEN,0);     // RX Mode
   sleep_us(20);
   gpio_put(TRX_CE,1);   

//   sleep_ms(2000);

// Do a dummy read to clear things so it will work from power-up
   gpio_put(TXEN,0);
   sleep_us(20);
   gpio_put(TRX_CE,0);   // Read Data

// Initialize output data buffer; first byte is Read RX Payload command
   spi_out_buffer[0] = 0x24;

// Get the RX data
   gpio_put(CSN905,0);   
// Write the output buffer to MOSI, and at the same time read from MISO.
   spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, BUF_LEN);
   gpio_put(CSN905,1); 

   gpio_put(TXEN,0);   // RX Mode
   sleep_us(20);
   gpio_put(TRX_CE,1);  
 //-----------
   log_data();

   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen
}
//=============================================
void my_itoa(unsigned int bin,char *str)
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

// bin = 1234; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//===============================================
void my_itoa2(unsigned int bin,char *str)  // remove leading zeros; end with NULL
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

// bin = 1234; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
// Display it
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = 0;
   }
   else
   { 
      if(char2 != 0x30)
      { 
         str[0] = char2;
         str[1] = char3;
         str[2] = char4;
         str[3] = 0;
      }
      else
      {   
         if(char3 != 0x30)
         { 
            str[0] = char3;
            str[1] = char4;
            str[2] = 0;
         }
         else
         {
            str[0] = char4;
            str[1] = 0;
         }
      }
   }
}
//===============================================
// Fills a string with two ASCII numbers from two BCD numbers; ends with NULL if desired
void line_fill(unsigned char hex, char *ptr, unsigned char index, unsigned char end)
{
   unsigned char digit1, digit2;

   digit1 =  hex>>4;
   digit2 = hex & 0x0F;

   if(digit1<10)
      {digit1 = digit1 + 0x30;}
   else 
      { digit1 = digit1 + 0x37;}

   if(digit2<10)  
      {digit2 = digit2 + 0x30;}
   else 
      { digit2 = digit2 + 0x37;}

      ptr[index++] = digit1;
      ptr[index] = digit2;
      if(end) { ptr[++index] = 0; }  // NULL
}
//================================================      
// Fills a string with two ASCII numbers from a binary byte; ends with NULL if desired
void line_fill_bin(unsigned char bin,char *ptr, unsigned char index, unsigned char end)
{
// First Convert unsigned hex byte to BCD 
// Adapted from 8-bit microcontroller code with a DAA instruction
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char char1, char2;
   unsigned char hcarry;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
//          hcarry = 1;
      }
      else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Put it in the string
   ptr[index++] = char1;
   ptr[index] = char2;
   if(end) { ptr[++index] = 0; }  // NULL
}
//====================================
// send data out through uart for logging the data, 32 hex bytes
void log_data()
{
   unsigned char n;

   line[0] = 0;
   for(n=1; n<33; n++)
   {
      my_hextoa2(spi_in_buffer[n],nstr);
      strcat(line,nstr);  
 
      strcpy(line,"   ");  
      if(n==16) {strcat(line,"\n");} 
      else {strcpy(line,"   ");}  
   }
   strcpy(line,"\n\n");
   uart_puts(UART_ID,line);  
}
//====================================================
// send data out through uart for logging the data, 32 hex bytes; with time stamp
void log_data_ts()
{
   unsigned char n;

   log_time();  // time stamp in 'line'
   strcat(line,"\n");

   for(n=1; n<33; n++)
   {
      my_hextoa2(spi_in_buffer[n],nstr);
      strcat(line, nstr);  // add it to the line
  
      if(n==16) {strcat(line,"\n");}  
      else {strcat(line,"   ");}
   }
   strcat(line,"\n\n");
   uart_puts(UART_ID,line);  
}
//====================================================
void my_hextoa2(unsigned char bin,char *str) 
{
// First Convert Binary to ASCII
// Adapted from 8-bit microcontroller code with a DAA instruction

   unsigned char hex0;
   unsigned char hex1;
   unsigned char char1, char2;

   hex0 = bin>>4;
   hex1 = bin&0x0F;

// Convert to ascii
   if(hex0<10)
      {char1 = hex0 + 0x30;}
   else 
      { char1 = hex0+ 0x37;}

   if(hex1<10)
      {char2 = hex1 + 0x30;}
   else 
      { char2 = hex1+ 0x37;}

// Make the string
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;  // NULL
}
//================================================
void LED_break()
{
   while(1)  
   {
      gpio_put(LED,1);
      sleep_ms(250); 
      gpio_put(LED,0);
      sleep_ms(250); 
   }
}
//=======================================================
void log_time()
{
   strcpy(line,"            ");   

   i2c_get_time();

   line_fill(time_buffer[MONTH], nstr, 0, 1);
   strcpy(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[DATE], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[YEAR], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(time_buffer[HOURS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[MINS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[SECS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

//   strcat(line,"\n");
}
//========================================================
// first 10 bytes, in hex
void show_data_lcd()
{
   int n, address;

   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

   address = 0x80;  // lcd position
   for(n=1; n<11; n++)
   {
       // Convert to ascii and display it
       lcd_wr_hex(address,spi_in_buffer[n]);
       address = address + 3;
       if(n == 5)  {address = 0xC0;}    // next line
   }

}
//========================================================
